package de.unisaar.jfap.ex3;

import java.net.MalformedURLException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * POJO holding the details of a parsed URL.
 *
 * @author Bernd Kiefer, DFKI
 * @author Joerg Steffen, DFKI
 */
public class Url {

  // Static pattern that parses:
  // <scheme>://(optional user(optional : pass)@)
  // domainName(optional :<port>)/<path (maybe empty)>
  private static final Pattern URL_PATTERN =
      Pattern.compile(
          // schemeName ://
          "([-.0-9a-zA-Z]+)://"
              // username (: password (opt))@
              + "(?:([a-zA-Z0-9%]+)(?::([a-zA-Z0-9%]*))?@)?"
              // domainName (: port (opt)) /
              + "([0-9a-zA-Z.-]+)(?::([0-9]{1,5}))?/"
              // path
              + "(.*)");

  // URL parts as extracted from URL string
  private String scheme;
  private String userName;
  private String password;
  private String domainName;
  private int port;
  private List<String> path;

  /**
   * Create a new URL object from the given URL string.
   *
   * @param urlString the URL string to create the URL from
   * @throws MalformedURLException when parsing fails
   */
  public Url(String urlString) throws MalformedURLException {

    Matcher m = URL_PATTERN.matcher(urlString);
    if (!m.matches()) {
      throw new MalformedURLException(urlString);
    }
    this.scheme = m.group(1);
    this.userName = m.group(2);
    this.password = m.group(3);
    this.domainName = m.group(4);
    String portMatch = m.group(5);
    if (null == portMatch) {
      this.port = -1;
    } else {
      this.port = Integer.parseInt(portMatch);
    }
    // or shorter:
    // this.port = m.group(5) == null ? -1 : Integer.parseInt(m.group(5));
    if (m.group(6).isEmpty()) {
      this.path = Collections.emptyList();
    } else {
      this.path = Arrays.asList(m.group(6).split("/"));
    }
  }

  public String getScheme() {
    return this.scheme;
  }

  public String getUserName() {
    return this.userName;
  }

  public String getPassword() {
    return this.password;
  }

  public String getDomainName() {
    return this.domainName;
  }

  public int getPort() {
    return this.port;
  }

  public List<String> getPath() {
    return this.path;
  }

  @Override
  public String toString() {

    StringBuilder sb = new StringBuilder();
    sb.append(this.scheme).append("://");
    if (this.userName != null) {
      sb.append(this.userName);
      if (this.password != null) {
        sb.append(':').append(this.password);
      }
      sb.append('@');
    }
    sb.append(this.domainName);
    if (this.port != -1) {
      sb.append(':').append(this.port);
    }
    for (String elt : this.path) {
      sb.append('/').append(elt);
    }
    if (this.path.isEmpty()) {
      sb.append('/');
    }
    return sb.toString();
  }
}
