package de.unisaar.jfap.ex3;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatExceptionOfType;

import java.net.MalformedURLException;
import org.junit.jupiter.api.Test;

class UrlTest {

  @Test
  void testSimpleUrl() throws MalformedURLException {
    String urlString = "https://docs.oracle.com/en/java/javase/17/docs/api/index.html";
    Url url = new Url(urlString);
    assertThat(url.getScheme()).isEqualTo("https");
    assertThat(url.getDomainName()).isEqualTo("docs.oracle.com");
    assertThat(url.getPath())
        .containsExactly("en", "java", "javase", "17", "docs", "api", "index.html");
    assertThat(url.getUserName()).isNull();
    assertThat(url.getPassword()).isNull();
    assertThat(url.getPort()).isEqualTo(-1);
    assertThat(url.toString()).isEqualTo(urlString);
  }

  @Test
  void testUrlWithUserName() throws MalformedURLException {
    String urlString = "https://foo@docs.oracle.com/en/java/javase/17/docs/api/index.html";
    Url url = new Url(urlString);
    assertThat(url.getScheme()).isEqualTo("https");
    assertThat(url.getDomainName()).isEqualTo("docs.oracle.com");
    assertThat(url.getPath())
        .containsExactly("en", "java", "javase", "17", "docs", "api", "index.html");
    assertThat(url.getUserName()).isEqualTo("foo");
    assertThat(url.getPassword()).isNull();
    assertThat(url.getPort()).isEqualTo(-1);
    assertThat(url.toString()).isEqualTo(urlString);
  }

  @Test
  void testUrlWithUserNamePassword() throws MalformedURLException {
    String urlString = "https://foo:bar@docs.oracle.com/en/java/javase/17/docs/api/index.html";
    Url url = new Url(urlString);
    assertThat(url.getScheme()).isEqualTo("https");
    assertThat(url.getDomainName()).isEqualTo("docs.oracle.com");
    assertThat(url.getPath())
        .containsExactly("en", "java", "javase", "17", "docs", "api", "index.html");
    assertThat(url.getUserName()).isEqualTo("foo");
    assertThat(url.getPassword()).isEqualTo("bar");
    assertThat(url.getPort()).isEqualTo(-1);
    assertThat(url.toString()).isEqualTo(urlString);
  }

  @Test
  void testUrlWithUserNamePasswordPort() throws MalformedURLException {
    String urlString = "https://foo:bar@docs.oracle.com:8080/en/java/javase/17/docs/api/index.html";
    Url url = new Url(urlString);
    assertThat(url.getScheme()).isEqualTo("https");
    assertThat(url.getDomainName()).isEqualTo("docs.oracle.com");
    assertThat(url.getPath())
        .containsExactly("en", "java", "javase", "17", "docs", "api", "index.html");
    assertThat(url.getUserName()).isEqualTo("foo");
    assertThat(url.getPassword()).isEqualTo("bar");
    assertThat(url.getPort()).isEqualTo(8080);
    assertThat(url.toString()).isEqualTo(urlString);
  }

  @Test
  void testUrlWithNoPath() throws MalformedURLException {
    String urlString = "https://docs.oracle.com/";
    Url url = new Url(urlString);
    assertThat(url.getScheme()).isEqualTo("https");
    assertThat(url.getDomainName()).isEqualTo("docs.oracle.com");
    assertThat(url.getPath()).isEmpty();
    assertThat(url.getUserName()).isNull();
    assertThat(url.getPassword()).isNull();
    assertThat(url.getPort()).isEqualTo(-1);
    assertThat(url.toString()).isEqualTo(urlString);
  }

  @Test
  void testUrlWithPercentEncodedChars() throws MalformedURLException {
    String urlString = "https://f%2Boo:b%20ar@docs.oracle.com/en/java/javase/17/docs/api/index.html";
    Url url = new Url(urlString);
    assertThat(url.getScheme()).isEqualTo("https");
    assertThat(url.getDomainName()).isEqualTo("docs.oracle.com");
    assertThat(url.getPath())
        .containsExactly("en", "java", "javase", "17", "docs", "api", "index.html");
    assertThat(url.getUserName()).isEqualTo("f%2Boo");
    assertThat(url.getPassword()).isEqualTo("b%20ar");
    assertThat(url.getPort()).isEqualTo(-1);
    assertThat(url.toString()).isEqualTo(urlString);
  }

  @Test
  void testUrlWithMissingScheme() {
    String urlString = "//docs.oracle.com/";
    assertThatExceptionOfType(MalformedURLException.class)
        .isThrownBy(() -> new Url(urlString))
        .withMessage(urlString);
  }

  @Test
  void testUrlWithMissingColon() {
    String urlString = "https//docs.oracle.com/";
    assertThatExceptionOfType(MalformedURLException.class)
        .isThrownBy(() -> new Url(urlString))
        .withMessage(urlString);
  }

  @Test
  void testUrlWithMissingDomain() {
    String urlString = "https://foo:bar:8080";
    assertThatExceptionOfType(MalformedURLException.class)
        .isThrownBy(() -> new Url(urlString))
        .withMessage(urlString);
  }
}
